==> web-ui-pack-0.4.1.tgz: package/controls/date.js <==
import dateCopyTime from "../helpers/dateCopyTime";
import dateFromString from "../helpers/dateFromString";
import dateToString from "../helpers/dateToString";
import localeInfo from "../objects/localeInfo";
import WUPBaseComboControl from "./baseCombo";
import WUPCalendarControl from "./calendar";
/* c8 ignore next */
/* istanbul ignore next */
!WUPCalendarControl && console.error("!"); // It's required otherwise import is ignored by webpack
const tagName = "wup-date";
/** Form-control with date picker
 * @tutorial Troubleshooting
 * * $options.format related only to displayed text, to work with other date-options like min/max use strict format 'YYYY-MM-DD'
 * @example
  const el = document.createElement("wup-date");
  el.$options.name = "dateOfBirthday";
  el.$initValue = "1990-10-24";
  el.$options.validations = { required: true };
  el.$options.format = "yyyy-MM-dd";
  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-date name="dateOfBirhday" utc initvalue="1990-10-24" min="1930-01-01" max="2010-01-01"/>
  </wup-form>;
 */
export default class WUPDateControl extends WUPBaseComboControl {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    #ctr = this.constructor;
    static get $styleRoot() {
        return `:root {
        --ctrl-date-icon-img: url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='768' height='768'%3E%3Cpath d='M652.801 76.8c42.24 0 76.8 34.56 76.8 76.8v537.601c0 42.24-34.56 76.8-76.8 76.8H115.2c-42.624 0-76.8-34.56-76.8-76.8l.384-537.601c0-42.24 33.792-76.8 76.416-76.8h38.4V0h76.8v76.8h307.2V0h76.8v76.8h38.4zM192 345.6h76.8v76.8H192v-76.8zm230.4 0v76.8h-76.8v-76.8h76.8zm153.601 0h-76.8v76.8h76.8v-76.8zM115.2 691.2h537.601V268.8H115.2v422.4z'/%3E%3C/svg%3E");
      }`;
    }
    static get $style() {
        return `${super.$style}
      :host {
        --ctrl-icon-img: var(--ctrl-date-icon-img);
      }
      :host wup-calendar {
        margin: 0;
      }
      :host [menu] {
        overflow: hidden;
      }`;
    }
    static get observedOptions() {
        const arr = super.observedOptions;
        arr.push("format");
        return arr;
    }
    static get observedAttributes() {
        const arr = super.observedAttributes;
        arr.push("format", "min", "max", "utc", "exclude");
        return arr;
    }
    static $defaults = {
        ...WUPBaseComboControl.$defaults,
        // debounceMs: 500,
        validationRules: {
            ...WUPBaseComboControl.$defaults.validationRules,
            min: (v, setV, c) => (v === undefined || v < setV) &&
                `Min value is ${dateToString(setV, c._opts.format.toUpperCase())}`,
            max: (v, setV, c) => (v === undefined || v > setV) &&
                `Max value is ${dateToString(setV, c._opts.format.toUpperCase())}`,
            exclude: (v, setV) => (v === undefined || setV.some((d) => d.valueOf() === v.valueOf())) && `This value is disabled`,
        },
        // firstWeekDay: 1,
        // format: localeInfo.date.toLowerCase()
    };
    // @ts-expect-error reason: validationRules is different
    $options = {
        ...this.#ctr.$defaults,
        utc: true,
        format: this.#ctr.$defaults.format || localeInfo.date.toLowerCase(),
        firstWeekDay: this.#ctr.$defaults.firstWeekDay || localeInfo.firstWeekDay,
    };
    // @ts-expect-error reason: validationRules is different
    _opts = this.$options;
    /** Parse string to Date
     * @see WUPCalendarControl.$parse */
    parse(text) {
        /* istanbul ignore else */
        if (!text) {
            return undefined;
        }
        return WUPCalendarControl.$parse(text, !!this._opts.utc);
    }
    /** Called to parse input text to value (related to locale or pointed format)
     *  @tutorial Troubleshooting
     * * for "yyyy-mm-dd" the correct format is "yyyy-MM-dd" */
    parseInput(text) {
        const format = `${this._opts.format.toUpperCase()} hh:mm:ss.fff${this._opts.utc ? "Z" : ""}`;
        const v = dateFromString(text, format, { throwOutOfRange: true }) ?? undefined;
        return v;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    canParseInput(_text) {
        return !this.refMask || this.refMask.isCompleted;
    }
    gotChanges(propsChanged) {
        this._opts.utc = this.getBoolAttr("utc", this._opts.utc);
        this._opts.format = (this.getAttribute("format") ?? this._opts.format) || "YYYY-MM-DD";
        if (this._opts.format.toUpperCase().includes("MMM")) {
            console.error(`${this.tagName}. 'MMM' in format isn't supported`);
            this._opts.format = "YYYY-MM-DD";
        }
        this._opts.mask =
            this._opts.mask ??
                this._opts.format
                    .replace(/[yY]/g, "0")
                    .replace(/dd|DD/, "00")
                    .replace(/[dD]/, "#0")
                    .replace(/mm|MM/, "00")
                    .replace(/[mM]/, "#0"); // convert yyyy-mm-dd > 0000-00-00; d/m/yyyy > #0/#0/0000
        this._opts.maskholder = this._opts.maskholder ?? this._opts.format.replace(/([mMdD]){1,2}/g, "$1$1");
        this._opts.min = this.parse(this.getAttribute("min") || "") ?? this._opts.min;
        this._opts.max = this.parse(this.getAttribute("max") || "") ?? this._opts.max;
        this._opts.exclude = this.getRefAttr("exclude");
        const attr = this.getAttribute("startwith");
        if (attr != null) {
            // prettier-ignore
            switch (attr.toLowerCase()) {
                case "year":
                    this._opts.startWith = 2 /* PickersEnum.Year */;
                    break;
                case "month":
                    this._opts.startWith = 1 /* PickersEnum.Month */;
                    break;
                case "day":
                    this._opts.startWith = 0 /* PickersEnum.Day */;
                    break;
                default: delete this._opts.startWith;
            }
        }
        super.gotChanges(propsChanged);
    }
    get validations() {
        const vls = super.validations; // undefined impossible because of textControl || {};
        // user can type not valid value according to options min,max,exclude. So need to enable validations rules in this case
        if (this._opts.min)
            vls.min = this._opts.min;
        if (this._opts.max)
            vls.max = this._opts.max;
        if (this._opts.exclude)
            vls.exclude = this._opts.exclude;
        return vls;
    }
    goValidate(fromCase, silent = false) {
        // reset hours for validations
        const v = this.$value;
        const key = this._opts.utc ? "UTC" : "";
        const hh = v && [
            v[`get${key}Hours`](),
            v[`get${key}Minutes`](),
            v[`get${key}Seconds`](),
            v[`get${key}Milliseconds`](),
        ];
        v && v[`set${key}Hours`](0, 0, 0, 0);
        const r = super.goValidate(fromCase, silent);
        hh && v[`set${key}Hours`](hh[0], hh[1], hh[2], hh[3]);
        return r;
    }
    async renderMenu(popup, menuId) {
        popup.$options.minWidthByTarget = false;
        const el = document.createElement("wup-calendar");
        el.renderInput = () => {
            el.$refLabel.remove();
            return { menuId };
        };
        el.focusItem = (a) => this.focusMenuItem(a);
        el.focus = () => true; // to not allow to focus calendar itselft
        el.gotFocus.call(el);
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        el.setInputValue = () => { };
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        el.gotFormChanges = () => { }; // it's must be completely detached from formElement
        el.$refInput = this.$refInput;
        el.$options.validations = this.validations.required ? { required: true } : undefined;
        el.$options.startWith = this._opts.startWith;
        el.$options.exclude = this._opts.exclude;
        el.$options.max = this._opts.max;
        el.$options.min = this._opts.min;
        el.$options.utc = this._opts.utc;
        el.$options.name = undefined; // to detach from formElement
        el.$options.validationCase = 0; // disable any validations for control
        const v = this.$value;
        el.$initValue = v && !Number.isNaN(v.valueOf()) ? v : undefined;
        el.addEventListener("$change", (e) => {
            e.stopPropagation(); // otherwise date change event fired twice
            !el._isStopChanges && this.selectValue(el.$value);
        });
        popup.appendChild(el);
        return Promise.resolve(el);
    }
    setValue(v, canValidate = true, skipInput = false) {
        if (v && skipInput) {
            const prev = this.$value || this.$initValue;
            prev && dateCopyTime(v, prev, !!this._opts.utc); // copy time if was changing from input (calendar do it itself)
        }
        const isChanged = super.setValue(v, canValidate, skipInput);
        if (isChanged) {
            const c = this.$refPopup?.firstElementChild;
            if (c) {
                c._isStopChanges = true; // to prevent hidding popup by calendar valueChange
                c.$value = v && !Number.isNaN(v.valueOf()) ? v : undefined;
                setTimeout(() => (c._isStopChanges = false)); // without timeout calendar $changeEvent is fired
            }
        }
        return isChanged;
    }
    valueToInput(v) {
        if (v === undefined) {
            return "";
        }
        return dateToString(v, this._opts.format.toUpperCase() + (this._opts.utc ? "Z" : ""));
    }
    resetInputValue() {
        // don't call super because validation is appeared
    }
    gotInput(e) {
        super.gotInput(e, true);
    }
    gotKeyDown(e) {
        const wasOpen = this.$isOpen;
        let skipCalendar = false;
        switch (e.key) {
            case "Escape":
            case " ":
                skipCalendar = true;
                break;
            case "Home":
            case "End":
                skipCalendar = e.shiftKey || e.ctrlKey; // skip only if pressed shift + Home/End
                break;
            default:
                break;
        }
        const clnd = this.$refPopup?.firstElementChild;
        wasOpen && !skipCalendar && clnd.gotKeyDown.call(clnd, e); // skip actions for Escape & Space keys
        const r = !e.defaultPrevented && super.gotKeyDown(e);
        !wasOpen && this.$isOpen && !skipCalendar && clnd.gotKeyDown.call(clnd, e); // case when user press ArrowKey for opening menu
        return r || Promise.resolve();
    }
    focusMenuItem(next) {
        // WARN: it's important don't use call super... because the main logic is implemented inside calendar
        // can be fired from baseCombo => when need to clear selection
        const el = this.$refPopup?.firstElementChild;
        /* istanbul ignore else */
        if (el) {
            Object.getPrototypeOf(el).focusItem.call(el, next);
        }
        else if (!next) {
            this.$refInput.removeAttribute("aria-activedescendant");
        }
        this._focusedMenuItem = next;
    }
}
customElements.define(tagName, WUPDateControl);
// NiceToHave: role 'spinbutton" + changing input value via scrolling: https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Roles/spinbutton_role
// NiceToHave: allowYear, allowMonth, allowDays based on format: "YYYY-MM" - only for selection year & month
// NiceToHave: alt-behavior; when user press Alt allow to use arrowKeys to navigate in input - use logic for all comboboxes

==> web-ui-pack-0.8.1.tgz: package/controls/date.js <==
import dateCopyTime from "../helpers/dateCopyTime";
import dateFromString from "../helpers/dateFromString";
import dateToString from "../helpers/dateToString";
import localeInfo from "../objects/localeInfo";
import WUPBaseComboControl from "./baseCombo";
import WUPCalendarControl from "./calendar";
/* c8 ignore next */
/* istanbul ignore next */
!WUPCalendarControl && console.error("!"); // It's required otherwise import is ignored by webpack
const tagName = "wup-date";
/** Form-control with datepicker
 * @tutorial Troubleshooting
 * * $options.format related only to displayed text, to work with other date-options like min/max use strict format 'YYYY-MM-DD'
 * @example
  const el = document.createElement("wup-date");
  el.$options.name = "dateOfBirthday";
  el.$initValue = "1990-10-24";
  el.$options.validations = { required: true };
  el.$options.format = "yyyy-MM-dd";
  const form = document.body.appendChild(document.createElement("wup-form"));
  form.appendChild(el);
  // or HTML
  <wup-form>
    <wup-date name="dateOfBirhday" utc initvalue="1990-10-24" min="1930-01-01" max="2010-01-01"/>
  </wup-form>; */
export default class WUPDateControl extends WUPBaseComboControl {
    /** Returns this.constructor // watch-fix: https://github.com/Microsoft/TypeScript/issues/3841#issuecomment-337560146 */
    #ctr = this.constructor;
    static get $styleRoot() {
        return `:root {
        --ctrl-date-icon-img: url("data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='768' height='768'%3E%3Cpath d='M652.801 76.8c42.24 0 76.8 34.56 76.8 76.8v537.601c0 42.24-34.56 76.8-76.8 76.8H115.2c-42.624 0-76.8-34.56-76.8-76.8l.384-537.601c0-42.24 33.792-76.8 76.416-76.8h38.4V0h76.8v76.8h307.2V0h76.8v76.8h38.4zM192 345.6h76.8v76.8H192v-76.8zm230.4 0v76.8h-76.8v-76.8h76.8zm153.601 0h-76.8v76.8h76.8v-76.8zM115.2 691.2h537.601V268.8H115.2v422.4z'/%3E%3C/svg%3E");
      }`;
    }
    static get $style() {
        return `${super.$style}
      :host {
        --ctrl-icon-img: var(--ctrl-date-icon-img);
      }
      :host > [menu] {
        overflow: hidden;
      }
      :host > [menu] > wup-calendar {
        margin: 0;
      }`;
    }
    static get observedOptions() {
        const arr = super.observedOptions;
        arr.push("format");
        return arr;
    }
    static get observedAttributes() {
        const arr = super.observedAttributes;
        arr.push("format", "min", "max", "utc", "exclude");
        return arr;
    }
    static $defaults = {
        ...WUPBaseComboControl.$defaults,
        // debounceMs: 500,
        validationRules: {
            ...WUPBaseComboControl.$defaults.validationRules,
            min: (v, setV, c) => (v === undefined || v < setV) &&
                `Min value is ${dateToString(setV, c._opts.format.toUpperCase())}`,
            max: (v, setV, c) => (v === undefined || v > setV) &&
                `Max value is ${dateToString(setV, c._opts.format.toUpperCase())}`,
            exclude: (v, setV) => (v === undefined || setV.some((d) => d.valueOf() === v.valueOf())) && `This value is disabled`,
        },
        // firstWeekDay: 1,
        // format: localeInfo.date.toLowerCase()
        utc: true,
    };
    constructor() {
        super();
        this._opts.format = this.#ctr.$defaults.format || localeInfo.date.toLowerCase();
        this._opts.firstWeekDay = this.#ctr.$defaults.firstWeekDay || localeInfo.firstWeekDay; // init here to depends on localeInfo
    }
    /** Parse string to Date
     * @see {@link WUPCalendarControl.$parse} */
    parse(text) {
        if (!text) {
            return undefined;
        }
        return WUPCalendarControl.$parse(text, !!this._opts.utc);
    }
    /** Called to parse input text to value (related to locale or pointed format)
     *  @tutorial Troubleshooting
     * * for "yyyy-mm-dd" the correct format is "yyyy-MM-dd" */
    parseInput(text) {
        const format = `${this._opts.format.toUpperCase()} hh:mm:ss.fff${this._opts.utc ? "Z" : ""}`;
        const v = dateFromString(text, format, { throwOutOfRange: true }) ?? undefined;
        return v;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    canParseInput(_text) {
        return !this.refMask || this.refMask.isCompleted;
    }
    valueToUrl(v) {
        return dateToString(v, "yyyy-MM-dd");
    }
    gotChanges(propsChanged) {
        WUPCalendarControl.prototype.gotChangesSharable.call(this);
        this._opts.format = this.getAttr("format") || "YYYY-MM-DD";
        if (this._opts.format.toUpperCase().includes("MMM")) {
            this.throwError(`'MMM' in format isn't supported`);
            this._opts.format = "YYYY-MM-DD";
        }
        this._opts.mask =
            this._opts.mask ??
                this._opts.format
                    .replace(/[yY]/g, "0")
                    .replace(/dd|DD/, "00")
                    .replace(/[dD]/, "#0")
                    .replace(/mm|MM/, "00")
                    .replace(/[mM]/, "#0"); // convert yyyy-mm-dd > 0000-00-00; d/m/yyyy > #0/#0/0000
        this._opts.maskholder = this._opts.maskholder ?? this._opts.format.replace(/([mMdD]){1,2}/g, "$1$1");
        super.gotChanges(propsChanged);
    }
    get validations() {
        const vls = super.validations; // undefined impossible because of textControl || {};
        // user can type not valid value according to options min,max,exclude. So need to enable validations rules in this case
        if (this._opts.min)
            vls.min = this._opts.min;
        if (this._opts.max)
            vls.max = this._opts.max;
        if (this._opts.exclude)
            vls.exclude = this._opts.exclude;
        return vls;
    }
    goValidate(fromCase, silent = false) {
        // reset hours for validations
        const v = this.$value;
        const key = this._opts.utc ? "UTC" : "";
        const hh = v && [
            v[`get${key}Hours`](),
            v[`get${key}Minutes`](),
            v[`get${key}Seconds`](),
            v[`get${key}Milliseconds`](),
        ];
        v && v[`set${key}Hours`](0, 0, 0, 0);
        const r = super.goValidate(fromCase, silent);
        hh && v[`set${key}Hours`](hh[0], hh[1], hh[2], hh[3]);
        return r;
    }
    renderMenu(popup, menuId) {
        popup.$options.minWidthByTarget = false;
        const el = document.createElement("wup-calendar");
        el.renderInput = () => {
            el.$refLabel.remove();
            return { menuId };
        };
        el.focusItem = (a) => this.focusMenuItem(a);
        el.focus = () => true; // to not allow to focus calendar itselft
        el.gotFocus.call(el, new FocusEvent("focusin"));
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        el.setInputValue = () => { };
        // eslint-disable-next-line @typescript-eslint/no-empty-function
        el.gotFormChanges = () => { }; // it's must be completely detached from formElement
        el.$refInput = this.$refInput;
        el.$options.validations = this.validations.required ? { required: true } : undefined;
        el.$options.startWith = this._opts.startWith;
        el.$options.exclude = this._opts.exclude;
        el.$options.max = this._opts.max;
        el.$options.min = this._opts.min;
        el.$options.utc = this._opts.utc;
        el.$options.name = undefined; // to detach from formElement
        el.$options.validationCase = 0; // disable any validations for control
        const v = this.$value;
        el.$initValue = v && !Number.isNaN(v.valueOf()) ? v : undefined;
        el.setValue = (val, reason) => {
            const r = WUPCalendarControl.prototype.setValue.call(el, val, reason);
            if (reason === 4 /* SetValueReasons.userSelect */) {
                this.selectValue(el.$value);
            }
            return r;
        };
        el.$onChange = (e) => e.stopPropagation(); // otherwise date change event fired twice
        popup.appendChild(el);
        return el;
    }
    setValue(v, reason, skipInput = false) {
        if (v && skipInput) {
            const prev = this.$value || this.$initValue;
            prev && dateCopyTime(v, prev, !!this._opts.utc); // copy time if was changing from input (calendar do it itself)
        }
        const isChanged = super.setValue(v, reason, skipInput);
        if (isChanged) {
            const c = this.$refPopup?.firstElementChild;
            if (c) {
                c._isStopChanges = true; // to prevent hiding popup by calendar valueChange
                c.$value = v && !Number.isNaN(v.valueOf()) ? v : undefined;
                setTimeout(() => (c._isStopChanges = false)); // without timeout calendar $changeEvent is fired
            }
        }
        return isChanged;
    }
    valueToInput(v) {
        if (v === undefined) {
            return "";
        }
        return dateToString(v, this._opts.format.toUpperCase() + (this._opts.utc ? "Z" : ""));
    }
    resetInputValue() {
        // don't call super because validation is appeared
    }
    focusMenuItem(next) {
        // WARN: it's important don't use call super... because the main logic is implemented inside calendar
        // can be fired from baseCombo => when need to clear selection
        const el = this.$refPopup?.firstElementChild;
        if (el) {
            Object.getPrototypeOf(el).focusItem.call(el, next);
        }
        else {
            // if (!next)
            this.$refInput.removeAttribute("aria-activedescendant");
        }
        this._focusedMenuItem = next;
    }
    focusMenuItemByKeydown(e) {
        const clnd = this.$refPopup?.firstElementChild;
        clnd.gotKeyDown.call(clnd, e);
    }
}
customElements.define(tagName, WUPDateControl);
// NiceToHave: role 'spinbutton" + changing input value via scrolling: https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Roles/spinbutton_role
// NiceToHave: allowYear, allowMonth, allowDays based on format: "YYYY-MM" - only for selection year & month
// NiceToHave: alt-behavior; when user press Alt allow to use arrowKeys to navigate in input - use logic for all comboboxes
